@@ -92,6 +92,28 @@ gboolean InputThread::gtkplug_worker(GtkWidget *widget, GdkEvent *event, InputTh
 				case GDK_p:
 					th->m_sys->showProfilingData=!th->m_sys->showProfilingData;
 					break;
+				case GDK_m:
+					th->m_sys->audioManager->toggleMuteAll();
+					if(th->m_sys->audioManager->allMuted())
+						LOG(LOG_NO_INFO, "All sounds muted");
+					else
+						LOG(LOG_NO_INFO, "All sounds unmuted");
+					break;
+				case GDK_c:
+					if(th->m_sys->hasError())
+					{
+						GtkClipboard *clipboard;
+						clipboard = gtk_clipboard_get(GDK_SELECTION_CLIPBOARD);
+						gtk_clipboard_set_text(clipboard, th->m_sys->getErrorCause().c_str(),
+								th->m_sys->getErrorCause().size());
+						clipboard = gtk_clipboard_get(GDK_SELECTION_PRIMARY);
+						gtk_clipboard_set_text(clipboard, th->m_sys->getErrorCause().c_str(),
+								th->m_sys->getErrorCause().size());
+						LOG(LOG_NO_INFO, "Copied error to clipboard");
+					}
+					else
+						LOG(LOG_NO_INFO, "No error to be coppied to clipboard");
+					break;
 				default:
 					break;
 			}
@@ -203,6 +225,13 @@ void* InputThread::sdl_worker(InputThread* th)
 					case SDLK_p:
 						th->m_sys->showProfilingData=!th->m_sys->showProfilingData;
 						break;
+					case SDLK_m:
+						th->m_sys->audioManager->toggleMuteAll();
+						if(th->m_sys->audioManager->allMuted())
+							LOG(LOG_NO_INFO, "All sounds muted");
+						else
+							LOG(LOG_NO_INFO, "All sounds unmuted");
+						break;
 					case SDLK_q:
 						th->m_sys->setShutdownFlag();
 						if(th->m_sys->currentVm)
