@@ -20,6 +20,9 @@
 #ifndef COMPAT_H
 #define COMPAT_H
 
+// Set BOOST_FILEYSTEM_VERSION to 2 since boost-1.46 defaults to 3
+#define BOOST_FILESYSTEM_VERSION 2
+
 //Define cross platform helpers
 // TODO: This should be reworked to use CMake feature detection where possible
 
@@ -57,7 +60,7 @@ typedef int pid_t;
 #define be64toh(x) _byteswap_uint64(x)
 
 #include <malloc.h>
-inline int aligned_malloc(void **memptr, size_t alignment, size_t size)
+inline int aligned_malloc(void **memptr, std::size_t alignment, std::size_t size)
 {
 	*memptr = _aligned_malloc(size, alignment);
 	return (*memptr != NULL) ? 0: -1;
@@ -94,16 +97,21 @@ long lrint(double f);
 #define TLSDATA __thread
 #define CALLBACK
 
+//Support both atomic header ( gcc >= 4.6 ), and earlier ( stdatomic.h )
+#ifdef HAVE_ATOMIC
+#include <atomic>
+#else
 #include <stdatomic.h>
+#endif
 #define ATOMIC_INT32(x) std::atomic<int32_t> x
 #define ATOMIC_INCREMENT(x) x.fetch_add(1)
 #define ATOMIC_DECREMENT(x) (x.fetch_sub(1)-1)
 
 //Boolean type con acquire release barrier semantics
 #define ACQUIRE_RELEASE_FLAG(x) std::atomic_bool x
-#define ACQUIRE_READ(x) x.load(memory_order_acquire)
-#define RELEASE_WRITE(x, v) x.store(v, memory_order_release)
-int aligned_malloc(void **memptr, size_t alignment, size_t size);
+#define ACQUIRE_READ(x) x.load(std::memory_order_acquire)
+#define RELEASE_WRITE(x, v) x.store(v, std::memory_order_release)
+int aligned_malloc(void **memptr, std::size_t alignment, std::size_t size);
 void aligned_free(void *mem);
 #endif
 
