@@ -3821,6 +3821,12 @@ ASObject* GlobalObject::getVariableAndTargetByMultiname(const multiname& name, A
 	return ret;
 }*/
 
+ASFUNCTIONBODY(Boolean,generator)
+{
+	assert_and_throw(argslen==1);
+	return abstract_b(Boolean_concrete(args[0]));
+}
+
 //We follow the Boolean() algorithm, but return a concrete result, not a Boolean object
 bool lightspark::Boolean_concrete(ASObject* obj)
 {
